require 'spec_helper_acceptance'
describe 'cd4pe' do
  image = ENV['CD4PE_IMAGE']
  version = ENV['CD4PE_VERSION']
  cd4pe_image = image.nil? ? "" : "cd4pe_image => '#{image}',"
  cd4pe_version = version.nil? ? "" : "cd4pe_version => '#{version}',"

  manifest = <<-MANIFEST
    include docker
    docker::run { 'cd4pe_postgres':
      image                 => 'postgres:9.6',
      net                   => 'cd4pe',
      ports                 => ["5432:5432"],
      volumes               => ['cd4pe-postgres:/var/lib/postgresql/data'],
      env                   => ['POSTGRES_PASSWORD=puppetlabs'],
      health_check_interval => 10,
    }

    class { "cd4pe":
      #{cd4pe_image}
      #{cd4pe_version}
      manage_database => false,
      db_host         => 'cd4pe_postgres',
      db_name         => 'postgres',
      db_pass         => Sensitive('puppetlabs'),
      db_port         => 5432,
      db_provider     => 'postgres',
      db_user         => 'postgres',
      db_prefix       => 'test',
    }
  MANIFEST

  describe 'install' do
    context 'with postgres' do
      idempotent_apply(manifest)

      describe docker_container('cd4pe') do
        it { should exist }
        it { should be_running }
      end

      describe port(8080) do
        it { should be_listening }
      end

      describe port(8000) do
        it { should be_listening }
      end

      describe port(7000) do
        it { should be_listening }
      end
    end
  end
end
